
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include "dhcpd.h"
#include "dhcpc.h"

const char *dhcp_statestr(int state)
{
	switch (state) {
		case INIT_SELECTING:
			return "init_select";
		case REQUESTING:
			return "request";
		case BOUND:
			return "bound";
		case RENEWING:
			return "renew";
		case REBINDING:
			return "rebind";
		case INIT_REBOOT:
			return "init_reboot";
		case RENEW_REQUESTED:
			return "renew_requested";
		case RELEASED:
			return "release";
		default:
			return "unknown";
	}
}

const char *dhcp_msgstr(int msgtype)
{
	switch (msgtype) {
		case DHCPDISCOVER:
			return "discover";
		case DHCPOFFER:
			return "offer";
		case DHCPREQUEST:
			return "request";
		case DHCPDECLINE:
			return "decline";
		case DHCPACK:
			return "ack";
		case DHCPNAK:
			return "nak";
		case DHCPRELEASE:
			return "release";
		case DHCPINFORM:
			return "inform";
		default:
			return "unknown";
	}
}

#if (defined(DHCPD_DEBUG) || defined(DHCPC_DEBUG))


int enable_debug = 0;
const char *dbgfile = "/tmp/dhcp.debug";

int __debug(const char* func, int line, const char* str, ... )
{
	char buf[256];
	va_list arg;
	FILE *fp;

	memset(buf, '\0', 256);
	va_start(arg, str);
	vsnprintf(buf, 256, str, arg);
	va_end(arg);

	if ((fp = fopen(dbgfile,"a")) == NULL )
		return -1;
	fprintf(fp, "%s: ", func);
	fwrite(buf, strlen(buf), 1, fp);
	fprintf(fp,"\n");
	fclose(fp);
	
	if ((fp = fopen("/dev/console","w")) == NULL )
		return -1;
	fprintf(fp, "[%s - %d]:", func, line);
	fwrite(buf, strlen(buf), 1, fp);
	fprintf(fp,"\n");
	fclose(fp);

	return 0;
}


#endif
