/****************************************************************************
 *
 * SoftDslG993p2.h 
 *
 *
 * Description:
 *	This file contains definitions for G993p2 configuration
 *
 * $Revision: 1.12 $
 *
 * $Id: SoftDslG993p2.h,v 1.12 2007/08/24 00:35:05 rgreenf Exp $
 *
 * $Log: SoftDslG993p2.h,v $
 * Revision 1.12  2007/08/24 00:35:05  rgreenf
 * add latency path id
 *
 * Revision 1.11  2007/08/23 20:56:42  rgreenf
 * add extra framer parameter reporting
 *
 * Revision 1.10  2007/08/20 21:34:04  rgreenf
 * added training phase 6368 status reporting - ongoing
 *
 * Revision 1.9  2007/08/20 16:34:19  rgreenf
 * added training phase 6368 status reporting - ongoing
 *
 * Revision 1.8  2007/08/18 00:30:41  jboxho
 * PhyR implementation on 6368 - first step
 *
 * Revision 1.7  2007/08/11 00:44:55  ovandewi
 * PR31078: rework G993p2 common data pump capability struct
 *
 * Revision 1.6  2007/08/10 21:48:38  tonytran
 * Undo previous checked-in
 *
 * Revision 1.5  2007/08/10 16:16:01  tonytran
 * *** empty log message ***
 *
 * Revision 1.4  2007/08/05 19:00:53  ilyas
 * Added overhead message support to VDSL build and status definition for DBPrint
 *
 * Revision 1.3  2007/06/08 06:30:52  ilyas
 * Added API to get negotiated G.994 parameters
 *
 * Revision 1.2  2007/06/02 17:52:24  ilyas
 * Created firmware target for 6368, ongoing work
 *
 * Revision 1.1  2007/06/01 00:39:05  ilyas
 * Moved G993p2 definitions to a separate file for E14 code
 *
 *
 *****************************************************************************/

#ifndef	SoftDslG993p2Header
#define	SoftDslG993p2Header

/* Caution: Do not change anything in this structure definition, including associated constant */
/* This structure definition is used only by the driver and any change impose incompatibility issue in driver */
/* The structure following this structure (g993p2PhyDataPumpCapabilities) can be changed in PHY application */

#define NbandsSupport 5 /* FIXME - This is not the right place for the define - move to correct place */
#define 	MAX_XMT_NBANDS	5
#define 	MAX_RCV_NBANDS	5

#define		PROFILE8a 		0x1
#define		PROFILE8b 		0x2
#define		PROFILE8c 		0x4
#define		PROFILE8d 		0x8
#define		PROFILE12a 		0x10
#define		PROFILE12b 		0x20
#define		PROFILE17a 		0x40
#define		PROFILE30a 		0x80

typedef struct
	{
	 unsigned char 		selectedProfile ; 
	 short 				maxAggTxPwrDsQ4DB, maxAggTxPwrUsQ4DB ;  
	 unsigned char 		sprtUs0Reqd ; 
	 unsigned int 		minDataRate ; 
	 short 				profileDsMaxTone , profileUsMaxTone ; 
	 unsigned int 		maxDelay ; 
	 unsigned short		maxD ; 
	} g993p2ProfileContent; 

struct g993p2CommonDataPumpCapabilities
	{
	unsigned char		ADL, dsVirtualNoise, lineProbe, ldMode;
	unsigned short 		profileSel ;                              /* bitMap bit 0 -7 profileselect for 8a,8b,8c,8d,12a,12b,17a,30a respectively */

    short 				bandStartUs[MAX_XMT_NBANDS] ; 
	short 				bandStopUs[MAX_XMT_NBANDS] ; 
	short 				bandStartDs[MAX_RCV_NBANDS] ; 
	short 				bandStopDs[MAX_RCV_NBANDS] ; 
	short 				bandStartRFI[MAX_RCV_NBANDS] ; 
	short 				bandStopRFI[MAX_RCV_NBANDS] ; 

	short				nBandsUs;
	short				nBandsDs;
	short				nBandsRFI;
	
	unsigned char		IDFTsize ; 
	unsigned short 		initialIDFTsize , fillIFFT; 

	unsigned short 		ceLen_support ; 

    unsigned long		maskEU ;    /* 0->32, 1->36, 2->40, 3->44, 4->48, 5->52, 6->56, 7->60, 8->64 */
	unsigned long		maskADLU ;  /* 0->32, 1->36, 2->40, 3->44, 4->48, 5->52, 6->56, 7->60, 8->64 */
	unsigned long		maskA_US0; 	/* 0 -> A12b ; 1 -> A17a  ; */
	unsigned long		maskB_US0; 	/* 0 -> A12b ; 1 -> A17a  ; */

    unsigned short		subModePSDMasks;
	};

#ifdef BCM6368_SRC
typedef struct
	{
	struct g993p2CommonDataPumpCapabilities;
	} g993p2DataPumpCapabilities;


typedef struct
	{
	struct g993p2CommonDataPumpCapabilities;
	} g993p2PhyDataPumpCapabilities;
#else

typedef struct g993p2CommonDataPumpCapabilities g993p2DataPumpCapabilities;
typedef struct g993p2CommonDataPumpCapabilities g993p2PhyDataPumpCapabilities;

#endif

void *SoftDslGetSelectedG993p2Caps(void *gDslVars);
#ifdef FIRE_RETRANSMISSION
void *SoftDslGetPhyRSpecs(void *gDslVars);
#endif

/*
**
**			G.993 status flags and structures. 
**
**	  Status codes should be defined in SoftDsl.h under
**	  "General status messages (using clearEOC message structure)"
**	  starting from   kDslGeneralMsgStart
**
*/
#ifndef __KERNEL__
typedef struct {
	ulong		ctrl;
	ULONGLONG	arg1;
	ULONGLONG	arg2;
	ulong		lineId;
	ulong		txPh;
	ulong		txSym;
	ulong		rxPh;
	ulong		rxSym;
} e14DbPrintStatus;	
#endif

typedef struct ToneGroup ToneGroup;
struct ToneGroup
{
  unsigned short endTone;
  unsigned short startTone;
};

typedef struct Bp993p2 Bp993p2;
struct Bp993p2
{
  unsigned char  noOfToneGroups;
  unsigned char  reserved;
  ToneGroup toneGroups[16];
};


typedef struct Bpoint Bpoint;
struct Bpoint
{
  unsigned short tone;
  short value;
};

typedef struct Psd993p2 Psd993p2;
struct Psd993p2
{
  unsigned char      maxNbBP;
  unsigned char      n;
  Bpoint bp[32];
};

typedef struct UDenNum16 UDenNum16;
struct UDenNum16
{
  unsigned short num; /*numerator*/
  unsigned short den; /*denominator*/
};
typedef struct FramerDeframerOptions FramerDeframerOptions;
struct FramerDeframerOptions
{
  UDenNum16             S;          /* S== number of PMD symbols over which the FEC data frame spans (=1 for G.dmt fast path, <=1 for ADSL2 fast path) */
  unsigned short        D;          /* interleaving depth: =1 for fast path */
  unsigned short        N;          /* RS codeword size*/
  unsigned short        L;          /* Number of bits per symbol */        
  unsigned short        B;          /* nominal total of each bearer's octets per Mux Data Frame (Slightly redundant)*/
  unsigned short        U;          /* VDSL2: Number of OH sub-frames per OH frame */
  unsigned char         I;          /* VDSL2: Interleaver block length */
  unsigned char         R;          /* RS codeword overhead bytes */
  unsigned char         M;          /* ADSL2 only. Number of Mux frames per FEC Data frame.*/
  unsigned char         T;          /* ADSL2: Number of Mux frames per sync octet*/
                                    /* VDSL2: Number of Mux data frames in an OH sub-frame */
  unsigned char         G;          /* VDSL2: Notional number of OH bytes in an OH sub-frame - actual number of bytes in any sub-frame may 1 be greater than this */
  unsigned char         F;          /* VDSL2: Number of OH frames in an OH superframe */
  unsigned char         codingType; /* codingType associated with this option */
  unsigned char         fireEnabled;/* bitmap flagging fire support for this direction
                                     * and/or for reverse direction */
  unsigned char         fireRxQueue;/* length of the retransmission queue in Rx direection */
  unsigned char         tpsTcOptions; /* result of the pmsTcNegotiation */
  unsigned char         delay;      /* actual delay incurred on this latency path */
  unsigned char         INP;        /* actual INP guaranteed on this latency path */
  unsigned char         B1;
  unsigned char					ovhType;
  unsigned char         path;
  unsigned char					ahifChanId[2];
  unsigned char         tmType[2];
};


#endif	/* SoftDslG993p2Header */
