/*
<:copyright-broadcom

 Copyright (c) 2002 Broadcom Corporation
 All Rights Reserved
 No portions of this material may be reproduced in any form without the
 written permission of:
          Broadcom Corporation
          16215 Alton Parkway
          Irvine, California 92619
 All information contained in this document is Broadcom Corporation
 company private, proprietary, and trade secret.

:>
*/
//**************************************************************************
// File Name  : BcmAtmBApi.h
//
// Description: This file contains the definitions, structures and function
//              prototypes for the Broadcom Asynchronous Transfer Mode (ATM)
//              Bonding Application Program Interface (API).
//              Used by the user layers as well as the ATMAPI modules.
//
// Updates    : 02/23/2006  lat.  Created.
//**************************************************************************

#if !defined(_BCMATMBAPI_H_)
#define _BCMATMBAPI_H_

#if defined(__cplusplus)
extern "C" {
#endif

#include "bcmatmapi.h"


#ifdef BCM_ATM_BONDING

#define MAX_BONDED_LINES     4  /* ToDo */

typedef BCMATM_STATUS  BCMATMB_STATUS ;
typedef UINT8 *AllocFunction(int requestedSize) ;
typedef void   FreeFunction(UINT8 *bufferp) ;

typedef struct _MainBufferHeader {
  struct _MainBufferHeader *nextBuffer ;
  UINT16 allocatedSmallBuffer ;
  UINT16 dummy2 ;
  UINT16 dummy3 ;
  UINT16 applicationFirst ;
  UINT16 applicationSecond ;
} MainBufferHeader ;

typedef struct _AtmBRcvBufParms {
   /* Config */
   UINT32           ulFreePktQSize ;
   UINT16           usFreePktQBufSize ;
   UINT16           usFreePktQBufOffset ;
   AllocFunction    *pFnMainBufAlloc ;
   FreeFunction     *pFnMainBufFree ;
   UINT16           mainBufferNumber ;
   UINT16           mainBufferOffset ;

   /* Status */
   UINT16           mainBuffSize ;
   UINT16           smallBuffSize ;
   char             *pPktQMem ;
   UINT32           ulFreePktQBufSizeExp ;
   MainBufferHeader *pFirstAllocBuff ;
   MainBufferHeader *pLastAllocBuff ;
   UINT32           ulReceivePktQSize ;
} AtmBRcvBufParams ;

typedef struct _AtmBMgmtStats {
   UINT32 bonding_group_id;
   UINT32 adsl_aggr_rate_us;
   UINT32 adsl_aggr_rate_ds;
   UINT32 asm_rx_count[MAX_BONDED_LINES];
   UINT32 asm_tx_count[MAX_BONDED_LINES];
   UINT32 adsl_rate_us[MAX_BONDED_LINES];
   UINT32 adsl_rate_ds[MAX_BONDED_LINES];
   UINT32 state_changes_count[MAX_BONDED_LINES];
   UINT32 local_state_changes_count[MAX_BONDED_LINES];
   UINT32 local_rx_state[MAX_BONDED_LINES];
   UINT32 local_tx_state[MAX_BONDED_LINES];
   UINT32 remote_rx_state[MAX_BONDED_LINES];
   UINT32 remote_tx_state[MAX_BONDED_LINES];
   UINT32 remote_id[MAX_BONDED_LINES];
   UINT32 polled ;
   UINT32 responded ;
} AtmBMgmtStats ;

typedef struct _AtmBStats {
   UINT32   resetStats;    /* reset statistics */

   UINT32   rxCtlPkts ;    /* Total ASMs received */
   UINT32   rxPkts ;       /* total packets received */
   UINT32   rxAtm ;
   UINT32   rxBytes ;

   /* Bonding Stats */
   UINT32   outOfOrder ;
   UINT32   outOfOrderCompleteFlush ;
   UINT32   outOfRangeSid ;
   UINT32   missingCell ;
   UINT32   smallNegative ;

   /* detailed rx_errors: */
   UINT32   rxFifoErrors ;     /* recv'r fifo overrun      */
   UINT32   rxVpVcErrors ;
} AtmBStats ;


typedef struct AtmCell {
   UINT8      celldata[56]; /* round up the ATM cell size. With 4 bytes header,
                             * it could be sufficient to use 52 bytes (48+4) */
} AtmCell ;

typedef struct AtmCellEncaps {
   UINT32  cellOffset; /* offset of the cell compared to AtmCellEncaps */
   UINT32  sidOffset;  /* offset of the sid compared to the sid of the first cell
                        * in the buffer -- ~0 means ASM cell */
   AtmCell cell ;      /* 56 bytes */
   UINT32  portId ;
   UINT32  dummy1 ;
} AtmCellEncaps ;

typedef struct MultiAtmCellHeader {
   UINT8 dummy1 [15] ;
   UINT8 cellQuantity; /* number of effective non asm cells in the buffer */
   UINT8 totalCells;   /* determine the exact message size = (lastCellIdx+1)*68+20 */
   UINT8 asmIndex [3]; /* ~0 means no asm in buffer. 1 for each port of operation. */
   UINT16 firstSid;    /* provide 64 bit align */
   UINT16 maxSidOffet; /* provide 64 bit align */
} MultiAtmCellHeader ;

typedef struct
{
    UINT32              ulInterfaceId ;
    ATM_INTERFACE_CFG   InterfaceCfg ;
    BCMATMB_STATUS      baStatus ;
} ATMB_INTERFACE_CFG, *PATMB_INTERFACE_CFG ;

typedef struct
{
    ATM_VCC_ADDR    VccAddr ;
    UINT32          operStatus ;
    BCMATMB_STATUS  baStatus ;
} ATMB_VCC_CFG, *PATMB_VCC_CFG ;

typedef struct
{
    AtmBStats       Statistics ;
    BCMATMB_STATUS  baStatus ;
} ATMB_STATISTICS, *PATMB_STATISTICS ;

typedef struct
{
    AtmBMgmtStats   Statistics ;
    BCMATMB_STATUS  baStatus ;
} ATMB_MGMT_STATISTICS, *PATMB_MGMT_STATISTICS ;

//**************************************************************************
// Function Prototypes
//**************************************************************************

BCMATMB_STATUS BcmAtmb_Initialize( AtmBRcvBufParams * ) ;
BCMATMB_STATUS BcmAtmb_Uninitialize( void );
void BcmAtmb_SetInterfaceCfg( PATMB_INTERFACE_CFG pInterfaceCfg );
void BcmAtmb_SetVccCfg( PATMB_VCC_CFG pCfg );
void BcmAtmb_GetStatistics( PATMB_STATISTICS pStats) ;
void BcmAtmb_GetMgmtStatistics( PATMB_MGMT_STATISTICS pStats) ;
void BcmAtmb_ProcessIfTransparentData ( UINT32 ulHandle, PATM_VCC_ADDR pVccAddr,
    											    PATM_VCC_DATA_PARMS pDp, UINT32 ulInterfaceId) ;

UINT8* BcmAtmb_AllocSendBuffer (int size) ;
void   BcmAtmb_FreeSendBuffer (UINT8 *buffer) ;

#if defined(__cplusplus)
}
#endif
#endif /* #ifdef BCM_ATM_BONDING */

#endif // _BCMATMBAPI_H_

